package mathvis.ws06.jtem.ode;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import mathvis.ws06.jtem.foo.HeatEquation;

import de.jreality.swing.ScenePanel;
import de.jreality.ui.viewerapp.ViewerApp;
import de.jreality.vr.ViewerVR;
import de.jtem.jgimmick.SpinnerTimerControl;
import de.jtem.numericalMethods.calculus.odeSolving.Extrap;
import de.jtem.numericalMethods.calculus.odeSolving.RungeKutta;
import de.jtem.numericalMethods.calculus.odeSolving.RungeKuttaFehlberg;

public class HeightFieldVR {

	EditableHeightField ehf;
	HeightFieldODE ode;
	ViewerVR vr;
	ScenePanel scenePanel;

	public HeightFieldVR() throws IOException {
		initScenePanel();
		initViewerVR();
		setSize(100, 100);
	}

	private void initScenePanel() {
		final JSpinner dtSpinner = new JSpinner(new SpinnerNumberModel(0.01, 1E-6, 1, 0.001));
		dtSpinner.setPreferredSize(new Dimension(60,20));
		scenePanel = new ScenePanel();
		scenePanel.setZOffset(-2.0);
		JFrame f = scenePanel.getFrame();
		JButton b = new JButton("Flow");
				
		final ActionListener step = new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						double[] xx = getHeightField().getValues();
						getODE().setHeightFieldValues(xx);
						double[] x = getODE().getOdeValues();

						RungeKuttaFehlberg.solve(getODE(), x, 0, ((Double)dtSpinner.getValue()), 1E-6);
						
						getHeightField().setValues(getODE().getHeightFieldValues());
					}
				};
		b.addActionListener(step);
		SpinnerNumberModel sm = new SpinnerNumberModel(1., 0., 1.E9, 1.);
		SpinnerTimerControl stc = new SpinnerTimerControl(sm);
		stc.remove(0);
		stc.getTimer().setDelay(5);
		sm.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				step.actionPerformed(null);
			}
		});
		f.getContentPane().setLayout(new FlowLayout());
		f.getContentPane().add(stc);
		f.getContentPane().add(b);
		f.getContentPane().add(dtSpinner);
		f.pack();
	}

	private void initViewerVR() throws IOException {
		vr=new ViewerVR();
		vr.addAlignTab();
		vr.addAppTab();
		vr.addToolTab();
		vr.addEnvTab();
		vr.setOffset(1.5);
		vr.addAlignTab();
		vr.setShowPoints(true);
		vr.setPickVertices(true);
	}

	protected EditableHeightField getHeightField() {
		return ehf;
	}

	protected HeightFieldODE getODE() {
		return ode;
	}

	public void setSize(int u, int v) {
		ehf = new EditableHeightField(u, v);
		ehf.getRoot().addTool(scenePanel.getPanelTool());
		vr.setContent(ehf.getRoot());
		ode = new HeatEquation(getHeightField().getValues(), u, v);
	}
	
	public void display() {
		ViewerApp va = vr.display();
		va.update();
		va.display();
	}
	
	public static void main(String[] args) throws IOException {
		HeightFieldVR hfvr = new HeightFieldVR();
		hfvr.display();
	}
	
}
